module Typecheck where 

import Abssyntax
import Control.Monad.State
import Data.Map as Map
import Prelude hiding (lookup)


type Types = Map Ident Type

typeCheckExp :: (MonadState Types m) => Exp -> m Type
typeCheckExp (BExp e o e') = do
	TInt <- typeCheckExp e
	TInt <- typeCheckExp e'
	return TBool
typeCheckExp (OpExp e o e') = do
	TInt <- typeCheckExp e
	TInt <- typeCheckExp e'
	return TInt
typeCheckExp (EVar i) = typeCheckVar i
typeCheckExp (EAss i e) = do
	a <- typeCheckVar i
	b <- typeCheckExp e
	if a == b then return a else fail "FEL!"
typeCheckExp (EInt i) = return TInt
typeCheckExp (EBool b) = return TBool
typeCheckExp EReadI = return TInt
typeCheckExp EReadB = return TBool
typeCheckExp (ExpT t e) = do
	t2 <- typeCheckExp e
	if t == t2 then return t else fail "FEL!"
typeCheckExp EDefault = return NoType
typeCheckExp (EPost i op) = do
	TInt <- typeCheckVar i
	return TInt


typeCheckVar :: (MonadState Types m) => Ident -> m Type 
typeCheckVar i = do
	e <- get
	lookup i e

typeCheckStm :: (MonadState Types m) => Stm -> m Type 
typeCheckStm SNoop = return NoType
typeCheckStm (SExp e) = do 
	typeCheckExp e
	return NoType
typeCheckStm (SBlock ss) = do 
	mapM typeCheckStm ss
	return NoType
typeCheckStm (SIf e s s') = do
	TBool <- typeCheckExp e
	NoType <- typeCheckStm s
	NoType <- typeCheckStm s
	return NoType
typeCheckStm (SWhile e s) = do
	TBool <- typeCheckExp e
	NoType <- typeCheckStm s
	return NoType
typeCheckStm (SDecl t i e) = do
	t2 <- typeCheckExp e
	if t == t2 then do
		m <- get
		put (insert i t m)
		return NoType
		else fail $ "Illegal to assign an expression of type "++show t2++" to variable "++show i++" of type "++show t2
typeCheckStm (SPrint e) = do
	typeCheckExp e
	return NoType

